pub struct LazyKeyValue {
pub key: Vec<u8>,
pub value: LazyValue,
}Expand description
Row returned by LazyIter.
Fields
key: Vec<u8>User key bytes.
value: LazyValueValue handle that may defer reading blob bytes.
Implementations
Sourceimpl LazyKeyValue
impl LazyKeyValue
Sourcepub fn into_key_value_sync(self) -> Result<KeyValue>
pub fn into_key_value_sync(self) -> Result<KeyValue>
Reads any deferred value bytes synchronously and returns an owned row.
Sourcepub async fn into_key_value(self) -> Result<KeyValue>
pub async fn into_key_value(self) -> Result<KeyValue>
Reads any deferred value bytes asynchronously and returns an owned row.