RuntimeCapabilities

Struct RuntimeCapabilities 

pub struct RuntimeCapabilities { /* private fields */ }
Capabilities exposed by a selected runtime configuration.

Implementations

Source

impl RuntimeCapabilities

pub const fn background_threads(self) -> bool

Returns whether the runtime can spawn background threads.

pub const fn cooperative_tasks(self) -> bool

Returns whether the runtime can cooperatively run maintenance tasks.

pub const fn blocking_adapter(self) -> bool

Returns whether the runtime can adapt blocking storage work.

pub const fn cancellation_tokens(self) -> bool

Returns whether cancellation tokens are supported.

pub const fn task_join(self) -> bool

Returns whether spawned tasks can be joined.

pub const fn platform_async_io(self) -> bool

Returns whether at least one Trine storage operation uses platform async I/O.