pub struct CompactionTriggerStats {
pub trigger: CompactionTrigger,
pub runs: u64,
pub input_tables: u64,
pub output_tables: u64,
pub input_bytes: u64,
pub output_bytes: u64,
}Expand description
Per-trigger table and byte totals read and written by compaction.
Fields
trigger: CompactionTriggerPicker reason shared by the compaction inputs counted in this row.
runs: u64Number of compaction inputs selected for this reason.
input_tables: u64Number of input table files read for this reason.
output_tables: u64Number of output table files written for this reason.
input_bytes: u64Input table bytes read for this reason.
output_bytes: u64Output table bytes written for this reason.