PointValue

Struct PointValue 

pub struct PointValue { /* private fields */ }
Value returned by BucketReader::get.

A PointValue may own its bytes or keep a shared reference to bytes inside a decoded table block. Use as_bytes when you only need to inspect the value, or into_vec when you need owned bytes.

Implementations

Source

impl PointValue

pub fn as_bytes(&self) -> &[u8]

Returns the value bytes.

pub fn into_vec(self) -> Vec<u8>

Returns owned value bytes.

This is allocation-free when the value is already owned, and copies when the value is backed by shared table-block bytes.

pub fn into_value(self) -> Value

Returns an owned Value.

This is equivalent to into_vec.