pub struct RuntimeCapabilities { /* private fields */ }Expand description
Capabilities exposed by a selected runtime configuration.
Implementations
Sourceimpl RuntimeCapabilities
impl RuntimeCapabilities
Sourcepub const fn background_threads(self) -> bool
pub const fn background_threads(self) -> bool
Returns whether the runtime can spawn background threads.
Sourcepub const fn cooperative_tasks(self) -> bool
pub const fn cooperative_tasks(self) -> bool
Returns whether the runtime can cooperatively run maintenance tasks.
Sourcepub const fn blocking_adapter(self) -> bool
pub const fn blocking_adapter(self) -> bool
Returns whether the runtime can adapt blocking storage work.
Sourcepub const fn cancellation_tokens(self) -> bool
pub const fn cancellation_tokens(self) -> bool
Returns whether cancellation tokens are supported.
Sourcepub const fn platform_async_io(self) -> bool
pub const fn platform_async_io(self) -> bool
Returns whether at least one Trine storage operation uses platform async I/O.